IDI_ICON1               ICON    DISCARDABLE     "launchy.ico"

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 1,9,9,1
 PRODUCTVERSION 1,9,9,1
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x4L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904e4"
        BEGIN
            VALUE "CompanyName", "Code Jelly"
            VALUE "FileDescription", "Launchy"
            VALUE "FileVersion", "1.0.0"
            VALUE "InternalName", "Launchy.exe"
            VALUE "LegalCopyright", "This is GNU Software copyright Josh Karlin"
            VALUE "OriginalFilename", "Launchy.exe"
            VALUE "ProductName", "Launchy"
            VALUE "ProductVersion", "2.0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1252
    END
END